@@ -16,10 +16,11 @@
 #
 
 from py4j.java_gateway import JavaObject
+from py4j.java_collections import JavaArray
 from pyspark.ml.linalg import DenseVector, DenseMatrix
 from pyspark.ml.param import TypeConverters
 from pyspark.ml.util import _jvm
-from pyspark.sql import DataFrame
+from pyspark.sql import DataFrame, SparkSession
 
 
 class H2OTypeConverters(object):
@@ -66,6 +67,17 @@ class H2OTypeConverters(object):
 
         return convert
 
+    @staticmethod
+    def toList(value):
+        if value is None:
+            raise TypeError("None is not allowed.")
+        else:
+            valueForConversion = value
+            if isinstance(value, JavaObject):
+                valueForConversion = list(value)
+
+            return TypeConverters.toList(valueForConversion)
+
     @staticmethod
     def toListString():
         def convert(value):
@@ -256,7 +268,7 @@ class H2OTypeConverters(object):
             if value is None:
                 raise TypeError("None is not allowed.")
             else:
-                return [H2OTypeConverters.toListFloat()(v) for v in TypeConverters.toList(value)]
+                return [H2OTypeConverters.toListFloat()(v) for v in H2OTypeConverters.toList(value)]
 
         return convert
 
@@ -276,7 +288,7 @@ class H2OTypeConverters(object):
             if value is None:
                 raise TypeError("None is not allowed.")
             else:
-                return [H2OTypeConverters.toListString()(v) for v in TypeConverters.toList(value)]
+                return [H2OTypeConverters.toListString()(v) for v in H2OTypeConverters.toList(value)]
 
         return convert
 
@@ -390,6 +402,18 @@ class H2OTypeConverters(object):
 
         return convert
 
+    @staticmethod
+    def toNullableListJavaObject():
+        def convert(value):
+            if value is None:
+                return None
+            elif isinstance(value, list) or isinstance(value, JavaArray):
+                return [H2OTypeConverters.toJavaObj()(obj) for obj in value]
+            else:
+                raise TypeError("Invalid type: " + str(type(value)))
+
+        return convert
+
     @staticmethod
     def toDictionaryWithAnyElements():
         def convert(value):
@@ -479,7 +503,7 @@ class H2OTypeConverters(object):
         if value is None:
             return None
         else:
-            H2OTypeConverters.scalaMapStringStringToDictStringAny(value)
+            return H2OTypeConverters.scalaMapStringStringToDictStringAny(value)
 
     @staticmethod
     def scalaArrayToPythonArray(array):
@@ -489,3 +513,35 @@ class H2OTypeConverters(object):
             return [v for v in array]
         else:
             raise TypeError("Invalid type.")
+
+    @staticmethod
+    def scala2DArrayToPython2DArray(array):
+        if array is None:
+            return None
+        elif isinstance(array, JavaObject):
+            return [H2OTypeConverters.scalaArrayToPythonArray(v) for v in array]
+        else:
+            raise TypeError("Invalid type.")
+
+    @staticmethod
+    def scalaToPythonDataFrame(jdf):
+        if jdf is None:
+            return None
+        elif isinstance(jdf, JavaObject):
+            session = SparkSession.builder.getOrCreate()
+            if hasattr(session, '_wrapped'):
+                sqlContext = session._wrapped
+            else:
+                sqlContext = session  # Spark 3.3+ utilizes SparkSession instead of SQLContext
+            return DataFrame(jdf, sqlContext)
+        else:
+            raise TypeError("Invalid type.")
+
+    @staticmethod
+    def scalaDfArrayToPythonDfArray(array):
+        if array is None:
+            return None
+        elif isinstance(array, JavaObject):
+            return [H2OTypeConverters.scalaToPythonDataFrame(v) for v in array]
+        else:
+            raise TypeError("Invalid type.")
